/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * Copyright 2005-2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: panellib.c,v 1.3 2017/08/11 23:32:05 cyfm Exp $
 */

#include "ambient.h"

#if USE_PANEL_LIB 
/* public */
#include <exec/libraries.h>
#include <exec/execbase.h>
#include <dos/dos.h>
#if !USE_LEGACY
#include <libraries/query.h>
#endif
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/utility.h>

/* private */
#include "panellib.h"
#include "wbstart.h"
#include "mui_func.h"  
#include "copyright.h"
#include "smartreq.h"
#include "paneltags.h"
#include "gfx_bitmap.h"
#include "mui_func.h"

//extern APTR app;

struct Library *PanelLibBase;

static BPTR libexpunge(void);
static struct Library * LIB_Open(void);
static BPTR LIB_Close(void);
static BPTR LIB_Expunge(void);
static ULONG LIB_GetQueryAttr(void);

static ULONG LIB_SetPanelObjectBitMap(void);
static ULONG LIB_ObtainPanelObject(void);
static ULONG LIB_ReleasePanelObject(void);
static ULONG LIB_SetPanelObjectAttr(void);
static ULONG LIB_GetPanelObjectAttr(void);

static const ULONG libfunctable[] =
{
	FUNCARRAY_BEGIN,
	FUNCARRAY_32BIT_NATIVE,
	(ULONG)&LIB_Open,
	(ULONG)&LIB_Close,
	(ULONG)&LIB_Expunge,
	(ULONG)&LIB_GetQueryAttr,
	//(ULONG)&LIB_ArexxMustDie,
	
	(ULONG)&LIB_ObtainPanelObject,
	(ULONG)&LIB_ReleasePanelObject,
	(ULONG)&LIB_SetPanelObjectBitMap,
	(ULONG)&LIB_SetPanelObjectAttr,	
	(ULONG)&LIB_GetPanelObjectAttr,	
	0xffffffff,
	FUNCARRAY_END
};

extern struct ExecBase *SysBase;

static ULONG flush_panellib(void)
{
	ULONG retval = FALSE;
	struct Library *lib;
	
	/*
	 * Check if another alien library already exists
	 * and try to flush it.
	 */
	Forbid();
	if ( (lib = (struct Library *)FindName(&SysBase->LibList, "panel.library")) )
	{
		if (!lib->lib_OpenCnt)
		{
			RemLibrary(lib);

			if (!FindName(&SysBase->LibList, "panel.library"))
			{
				retval = TRUE;
			}
		}
	}
	else
	{
		retval = TRUE;
	}
	Permit();

	return (retval);
}


static const TEXT libname[] = "panel.library";
static const TEXT idstring[] = "panel.library 1.0 (11.10.2015)  2015 Ambient Open Source Team";

#if !USE_LEGACY
static const struct TagItem querytags[] = {
	{QUERYINFOATTR_NAME, (ULONG)libname},
	{QUERYINFOATTR_DESCRIPTION, (ULONG)"Library to interact with Ambient's panel system.."},
	{QUERYINFOATTR_COPYRIGHT, (ULONG)" 2016 by Ambient Open Source Team"},
	{QUERYINFOATTR_AUTHOR, (ULONG)"Ambient Open Source Team"},
	{QUERYINFOATTR_SUBTYPE, QUERYSUBTYPE_LIBRARY},
	{QUERYINFOATTR_CLASS, QUERYCLASS_NONE},
	{QUERYINFOATTR_SUBCLASS, QUERYSUBCLASS_NONE},
	{TAG_DONE, 0}
};
#endif

ULONG panellib_init(void)
{
	if (flush_panellib())
	{
		PanelLibBase = NewCreateLibraryTags(
			LIBTAG_FUNCTIONINIT, libfunctable,
			LIBTAG_BASESIZE, sizeof(struct Library),
			LIBTAG_TYPE, NT_LIBRARY,
			LIBTAG_NAME, libname,
			LIBTAG_FLAGS, LIBF_CHANGED | LIBF_SUMUSED
			#if USE_LEGACY
			,
			#else 
			| LIBF_QUERYINFO,
			#endif
			LIBTAG_VERSION, 1,
			LIBTAG_REVISION, 2,
			LIBTAG_IDSTRING, idstring,
			LIBTAG_PUBLIC, TRUE,
			TAG_DONE);
	}
	/* XXX: maybe add some debugging to know what happens.. */
	return (TRUE); /* we don't fail if some lib is already here. better than nothing */
}


void panellib_cleanup(void)
{
	if (PanelLibBase)
	{
		if (!libexpunge())
		{
			PDB(("should not happen (tm)\n")); /* XXX: actually it can but well .. */
		}
	}
}


static BPTR libexpunge(void)
{
	Forbid();
	if (PanelLibBase->lib_OpenCnt)
	{
		PanelLibBase->lib_Flags |= LIBF_DELEXP;
		Permit();
		return (0);
	}

	REMOVE(&PanelLibBase->lib_Node);
	Permit();

	FreeMem(((UBYTE *)PanelLibBase) - PanelLibBase->lib_NegSize,
	        PanelLibBase->lib_NegSize + PanelLibBase->lib_PosSize);

	return (TRUE);
}


static ULONG LIB_GetQueryAttr(void)
{
	#if !USE_LEGACY
	ULONG *data = (ULONG *)REG_A0;
	ULONG attr = REG_D0;

	if (data)
	{
		struct TagItem *ti;

		if ((ti = FindTagItem(attr, querytags)))
		{
			*data = ti->ti_Data;
			return (TRUE);
		}
	}
	#endif
	return (FALSE);
}


static struct Library * LIB_Open(void)
{
	PanelLibBase->lib_Flags &= ~LIBF_DELEXP;
	PanelLibBase->lib_OpenCnt++;

	return ((struct Library *)PanelLibBase);
}


static BPTR LIB_Expunge(void)
{
	return (0); /* do not let anything expunge us, except ourself */
}


static BPTR LIB_Close(void)
{
	if ((--PanelLibBase->lib_OpenCnt) == 0)
	{
		if (PanelLibBase->lib_Flags & LIBF_DELEXP)
		{
			return (0); /* same here */
		}
	}
	return (0);
}




static ULONG LIB_SetPanelObjectAttr(void)
{
	APTR panelobject = (APTR)REG_A0;
	if(panelobject)
	{
		//struct TagItem *tags = (struct TagItem *)REG_A1;
		struct TagItem *tag, *tagptr;
		tagptr = (struct TagItem *)REG_A1; 
		while ((tag = NextTagItem(&tagptr)))
		{
		//	DoMethod(app,MUIM_Application_PushMethod,panelobject,3,MUIM_Set,tag->ti_Tag,tag->ti_Data);
				PDB(("%x %d %d\n",panelobject,tag->ti_Tag,tag->ti_Data));
		}
	}

	return (0);
}

static ULONG LIB_GetPanelObjectAttr(void)
{
	APTR panelobject = (APTR)REG_A0;
	ULONG attrID = REG_D0;
	//ULONG *storage = (ULONG *)REG_A1;
	PDB(("%x %d\n",panelobject,attrID));
	if(panelobject)
	{
	//	DoMethod(app,MUIM_Application_PushMethod,panelobject,3,OM_GET,attrID,storage);
	}
	return (0);
}

static ULONG LIB_ObtainPanelObject(void)
{
	APTR win;//,child;
	ULONG type,open;
	struct TagItem /* *typeTag,*/*uriTag;
	APTR /*tbar,*/panelobject = 0;
	//APTR win_state,child_state;
	//struct List *win_list,*child_list;
	struct TagItem *tags = (struct TagItem *)REG_A0;
	STRPTR objectname = 0;
	//BPTR searchlock, objlock;

	if((uriTag = FindTagItem(PanelObjectURI,tags)))	objectname = (STRPTR) uriTag->ti_Data;
	//if((typeTag = FindTagItem(PanelObjectType,tags)))
	{
		//DoMethod(app,MUIM_Application_PushMethod,app,3,OM_GET, MUIA_Application_WindowList, (ULONG*) &win_list );
		PDB(("\n\n\n\n\n\n\n\n\n"));
		//win_state = win_list->lh_Head;
		while( 0)//( win = NextObject( &win_state ) ) && ( panelobject == 0 ) )
		{
			if( ( DoMethod(app,MUIM_Application_PushMethod,win,3,OM_GET, MA_Panelwin_Type, &type ) ) && ( type == MV_Panelwin_Type_Root ) )
			{
				PDB(("\n"));
				DoMethod(app,MUIM_Application_PushMethod,win,3,OM_GET,MUIA_Window_Open, &open );
				PDB(("\n"));
				if( open )
				{
					panelobject = (APTR)DoMethod(app,MUIM_Application_PushMethod,win,3,MM_Panelwin_FindObject,PanelObjectType_Command,objectname); 
					PDB(("\n"));
				}
			}
		}
	}
	PDB(("\n\n\n\n\n\n\n\n\n"));
	//if(panelobject)	DoMethod(app,MUIM_Application_PushMethod,panelobject,1,OM_RETAIN);											
	return (ULONG) panelobject;
}


static ULONG LIB_ReleasePanelObject(void)
{
	APTR panelobject = (APTR)REG_A0;
	if(panelobject)
	{
	//	DoMethod(app,MUIM_Application_PushMethod,panelobject,3,MUIM_Set,MA_Panelbutton_ExtraBitmap,NULL);
	//	DoMethod(app,MUIM_Application_PushMethod,panelobject,1,OM_RELEASE);
	}
	return 0;
}

static ULONG LIB_SetPanelObjectBitMap(void)
{
	APTR panelobject = (APTR)REG_A0;
	struct BitMap *bm = (struct BitMap*) REG_A1;
	ULONG width  = REG_D0;
	ULONG height = REG_D1;
	//struct TagItem *tags = (struct TagItem *)REG_A2;
	PDB(("%x %x %d %d\n",panelobject,bm,width,height));
	if(panelobject)
	{
		APTR abm; 
		abm = gfx_bitmap_create_from_native(bm,width,height);
		PDB(("%x\n",abm));
		//DoMethod(app,MUIM_Application_PushMethod,panelobject,3,MUIM_Set,MA_Panelbutton_ExtraBitmap,abm);
	}
	return 0;
}

ULONG preclose_panellib(void)
{
	if (PanelLibBase && PanelLibBase->lib_OpenCnt)
	{
		smartreq_request(NULL, NULL, NULL, 0, 0, "Ok", MV_Notification_Warning, "panel.library's opencount is %ld.\nClose the processes using it.", PanelLibBase->lib_OpenCnt);
		return (FALSE);
	}
	return (TRUE);
}
#endif /* USE_PANEL_LIB */ 
